# Copyright (c) 2018 gevent community
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
from __future__ import absolute_import, print_function, division

import importlib
import os.path
import warnings

import gevent

from . import sysinfo
from . import util


OPTIONAL_MODULES = frozenset({
    ## Resolvers.
    # ares might not be built
    'gevent.resolver_ares',
    'gevent.resolver.ares',
    # dnspython might not be installed
    'gevent.resolver.dnspython',
    ## Backends
    'gevent.libev',
    'gevent.libev.watcher',
    'gevent.libuv.loop',
    'gevent.libuv.watcher',
})

EXCLUDED_MODULES = frozenset({
    '__init__',
    'core',
    'ares',
    '_util',
    '_semaphore',
    'corecffi',
    '_corecffi',
    '_corecffi_build',
})

def walk_modules(
        basedir=None,
        modpath=None,
        include_so=False,
        recursive=False,
        check_optional=True,
        include_tests=False,
        optional_modules=OPTIONAL_MODULES,
        excluded_modules=EXCLUDED_MODULES,
):
    """
    Find gevent modules, yielding tuples of ``(path, importable_module_name)``.

    :keyword bool check_optional: If true (the default), then if we discover a
       module that is known to be optional on this system (such as a backend),
       we will attempt to import it; if the import fails, it will not be returned.
       If false, then we will not make such an attempt, the caller will need to be prepared
       for an `ImportError`; the caller can examine *optional_modules* against
       the yielded *importable_module_name*.
    """
    # pylint:disable=too-many-branches
    if sysinfo.PYPY:
        include_so = False
    if basedir is None:
        basedir = os.path.dirname(gevent.__file__)
        if modpath is None:
            modpath = 'gevent.'
    else:
        if modpath is None:
            modpath = ''

    for fn in sorted(os.listdir(basedir)):
        path = os.path.join(basedir, fn)
        if os.path.isdir(path):
            if not recursive:
                continue
            if not include_tests and fn in ['testing', 'tests']:
                continue
            pkg_init = os.path.join(path, '__init__.py')
            if os.path.exists(pkg_init):
                yield pkg_init, modpath + fn
                for p, m in walk_modules(
                        path, modpath + fn + ".",
                        include_so=include_so,
                        recursive=recursive,
                        check_optional=check_optional,
                        include_tests=include_tests,
                        optional_modules=optional_modules,
                        excluded_modules=excluded_modules,
                ):
                    yield p, m
            continue

        if fn.endswith('.py'):
            x = fn[:-3]
            if x.endswith('_d'):
                x = x[:-2]
            if x in excluded_modules:
                continue
            modname = modpath + x
            if check_optional and modname in optional_modules:
                try:
                    with warnings.catch_warnings():
                        warnings.simplefilter('ignore', DeprecationWarning)
                        importlib.import_module(modname)
                except ImportError:
                    util.debug("Unable to import optional module %s", modname)
                    continue
            yield path, modname
        elif include_so and fn.endswith(sysinfo.SHARED_OBJECT_EXTENSION):
            if '.pypy-' in fn:
                continue
            if fn.endswith('_d.so'):
                yield path, modpath + fn[:-5]
            else:
                yield path, modpath + fn[:-3]



# -*- coding: utf-8 -*-
"""
This file runs ``gevent.monkey.patch_all()``.

It is intended to be used by ``python -m gevent.monkey <this file>``
to prove that monkey-patching twice doesn't have unfortunate sife effects (such as
breaking the threadpool).
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import sys
from gevent import monkey
from gevent import get_hub

monkey.patch_all(thread=False, sys=True)

def thread_is_greenlet():
    from gevent.thread import get_ident as gr_ident
    std_thread_mod = 'thread' if bytes is str else '_thread'
    thr_ident = monkey.get_original(std_thread_mod, 'get_ident')
    return thr_ident() == gr_ident()


is_greenlet = get_hub().threadpool.apply(thread_is_greenlet)
print(is_greenlet)
print(len(sys._current_frames()))
sys.stdout.flush()
sys.stderr.flush()



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from gevent import lock


import gevent.testing as greentest
from gevent.tests import test__semaphore


class TestRLockMultiThread(test__semaphore.TestSemaphoreMultiThread):

    def _makeOne(self):
        # If we don't set the hub before returning,
        # there's a potential race condition, if the implementation
        # isn't careful. If it's the background hub that winds up capturing
        # the hub, it will ask the hub to switch back to itself and
        # then switch to the hub, which will raise LoopExit (nothing
        # for the background thread to do). What is supposed to happen
        # is that the background thread realizes it's the background thread,
        # starts an async watcher and then switches to the hub.
        #
        # So we deliberately don't set the hub to help test that condition.
        return lock.RLock()

    def assertOneHasNoHub(self, sem):
        self.assertIsNone(sem._block.hub)



if __name__ == '__main__':
    greentest.main()



import sys
import unittest

from gevent.testing import TestCase
import gevent
from gevent.timeout import Timeout

@unittest.skipUnless(
    hasattr(sys, 'gettotalrefcount'),
    "Needs debug build"
)
class TestQueue(TestCase): # pragma: no cover
    # pylint:disable=bare-except,no-member

    def test(self):
        result = ''
        try:
            Timeout.start_new(0.01)
            gevent.sleep(1)
            raise AssertionError('must raise Timeout')
        except KeyboardInterrupt:
            raise
        except:
            pass

        result += '%s ' % sys.gettotalrefcount()

        try:
            Timeout.start_new(0.01)
            gevent.sleep(1)
            raise AssertionError('must raise Timeout')
        except KeyboardInterrupt:
            raise
        except:
            pass

        result += '%s ' % sys.gettotalrefcount()

        try:
            Timeout.start_new(0.01)
            gevent.sleep(1)
            raise AssertionError('must raise Timeout')
        except KeyboardInterrupt:
            raise
        except:
            pass

        result += '%s' % sys.gettotalrefcount()

        _, b, c = result.split()
        assert b == c, 'total refcount mismatch: %s' % result



if __name__ == '__main__':
    unittest.main()
